//
// extensionApi.js
//
// This code provides the suExtensionApi implementation to the 
// stumbleupon.com web pages.
//
var su_extension_api = function(window) {
	this.id = null;
	this._siteWindow = window;

	// -------------------------------------------------------------------
	// Message support
	// -------------------------------------------------------------------
	// Note that this doesn't exist in the prototype because it has its own properties
	// that we don't want to share amongst other objects.
	this.message = {
		_api: this,
		_listeners: [],
		
		_handleIncoming: function(msg) {
			if(msg.target && (msg.target.id != this._api.id))
				return;
			// Send the message to the listeners.
			for(var i=0; i<this._listeners.length; i++)
			{
				this._listeners[i](msg.messageId, msg.data, msg.sender);
			}
		},
		
		addListener: function(fnListener) {
			this.removeListener(fnListener);
			this._listeners.push(fnListener);
		},
		
		removeListener: function(fnListener) {
			for(var i=0; i<this._listeners.length; i++)
			{
				if(this._listeners[i] === fnListener)
				{
					this._listeners.splice(i, 1);
					return;
				}
			}
		},
		
		broadcastMessage: function(messageId, data) {
			var sender = { id: this._api.id };
			var msg = { sender: sender, messageId: messageId, data: data };
			var obsService = Components.classes["@mozilla.org/observer-service;1"]
								.getService(Components.interfaces.nsIObserverService);
			obsService.notifyObservers(null, "su_extensionapi_message", su_ds.serialize(msg));
		},
		
		postMessage: function(target, messageId, data) {
			var sender = { id: this._api.id };
			var msg = { sender: sender, target: target, messageId: messageId, data: data };
			var obsService = Components.classes["@mozilla.org/observer-service;1"]
								.getService(Components.interfaces.nsIObserverService);
			obsService.notifyObservers(null, "su_extensionapi_message", su_ds.serialize(msg));
		}
	
	};
	
	this._init();
}

// -------------------------------------------------------------------
// Top-level su_extension_api methods
// -------------------------------------------------------------------
su_extension_api.prototype = {
	
	// -------------------------------------------------------------------
	// Core functionality
	// -------------------------------------------------------------------
	getProviderInfo: function(callback) {
		var info = {
			apiVersion: "1.0",
			provider: "mozbar"
		}
		info.providerVersion = mozbar_version + "." + su_private_label;
		callback(info);
	},
	
	getDomain: function(callback) {
		callback(su_servername);
	},

	oldbar: {
		processCommands: function(commands) {
			su_process_commands(commands);
		}
	},
	
	// -------------------------------------------------------------------
	// Data functionality
	// -------------------------------------------------------------------
	data: {
		_normalizeKey: function(key) {
			if(key.indexOf("@@") == 0)
			{
				key = key.slice(2);
			}
			else
			{
				key = "@extapi." + key;
			}
			return key;
		},
		
		getValue: function(key, callback) {
			var value = null;
			key = this._normalizeKey(key);
			if(su_ds.isPrefDefined(key))
				value = su_ds.getPrefValue(key, "JSON");
			callback(su_ds.deserialize(value));
		},
		
		setValue: function(key, value, callback) {
			key = this._normalizeKey(key);
			if(value == null)
			{
				su_ds.clearPref(key);
			}
			else
			{
				su_ds.setValue(key, value);
			}
			if(typeof(callback) != "undefined")
				callback();
		}
	},
	
	isReady: function() {
		return this._initialized;
	},

	// -------------------------------------------------------------------
	// Private implementation methods
	// -------------------------------------------------------------------
	
	// observer service callback.
	observe: function(subject, topic, data) 
	{
		if(topic == "su_extensionapi_message") 
		{
			msg = su_ds.deserialize(data);
			this.message._handleIncoming(msg);
		}
	},
	
	_generateEndpointId: function() {
		return Math.floor(Math.random() * 100000000);
	},
	
	// -------------------------------------------------------------------
	// Private initialization
	// -------------------------------------------------------------------
	_init: function() {
		if(this._initialized)
			return;
		
		// Generate a new endpoint id
		this.id = this._generateEndpointId();
		
		// Listen for messages using the observer service
		var obsService = Components.classes["@mozilla.org/observer-service;1"]
							.getService(Components.interfaces.nsIObserverService);
		obsService.addObserver(this, "su_extensionapi_message", false);
		
		// Listen for the window to unload so we can clean up
		var me = this;
		this._siteWindow.addEventListener("unload", function() {
			me._uninit();
		}, false);
		
		this._initialized = true;

		// Notify anyone who has been waiting for us to be ready.
		window.setTimeout(function() {
			var scriptReadyEvent = me._siteWindow.document.createEvent("Event");
			scriptReadyEvent.initEvent("suScriptReadyExtensionApi", false, false);
			me._siteWindow.dispatchEvent(scriptReadyEvent);
		}, 0);
	},
	
	_uninit: function() {
		// Remove any reference to ourselves
		this._siteWindow.suExtensionApi = null;

		// Remove our references to client listeners
		this.message._listeners = [];

		// Remove ourselves as an observer service listener
		var obsService = Components.classes["@mozilla.org/observer-service;1"]
							.getService(Components.interfaces.nsIObserverService);
		obsService.removeObserver(this, "su_extensionapi_message");
	}
}

